@@ -332,6 +332,9 @@ class SASconfigHTTP:
             self.ctx = contexts[i]
             break
 
+      if self.ctx == {}:
+         raise SASHTTPconnectionError(msg="No context information returned for context {}\n{}".format(self.ctxname, contexts))
+
       return
 
    def _authenticate(self, user, pw):
@@ -461,11 +464,16 @@ class SASsessionHTTP():
          options = '[]'
 
       # POST Session
+      uri = None
       for ld in self.sascfg.ctx.get('links'):
          if ld.get('method') == 'POST':
             uri = ld.get('uri')
             break
 
+      if not uri:
+         raise SASHTTPconnectionError(msg=
+         "POST uri not found in context info. You may not have permission to use this context.\n{}".format(self.sascfg.ctx))
+
       conn = self.sascfg.HTTPConn; conn.connect()
       d1 = '{"name":"'+self.sascfg.ctxname+'", "description":"saspy session", "version":1, "environment":{"options":'+options+'}}'
       headers={"Accept":"application/vnd.sas.compute.session+json","Content-Type":"application/vnd.sas.compute.session.request+json","Authorization":"Bearer "+self.sascfg._token}
@@ -530,7 +538,11 @@ class SASsessionHTTP():
       resp = req.read()
       conn.close()
 
-      jobid = json.loads(resp.decode(self.sascfg.encoding))
+      try:
+         jobid = json.loads(resp.decode(self.sascfg.encoding))
+      except:
+         jobid = None
+
       if not jobid or status > 299:
          print("Compute server had issues starting:\n")
          for key in jobid:
@@ -587,10 +599,12 @@ class SASsessionHTTP():
          resp = req.read()
          conn.close()
 
-         js  = json.loads(resp.decode(self.sascfg.encoding))
-         log = js.get('items')
-
-         lines = len(log)
+         try:
+            js    = json.loads(resp.decode(self.sascfg.encoding))
+            log   = js.get('items')
+            lines = len(log)
+         except:
+            lines = None
 
          if not lines:
             break
@@ -626,8 +640,11 @@ class SASsessionHTTP():
       resp = req.read()
       conn.close()
 
-      js = json.loads(resp.decode(self.sascfg.encoding))
-      results = js.get('items')
+      try:
+         js = json.loads(resp.decode(self.sascfg.encoding))
+         results = js.get('items')
+      except:
+         results = []
 
       conn = self.sascfg.HTTPConn; conn.connect()
       headers={"Accept":"application/vnd.sas.collection+json", "Authorization":"Bearer "+self.sascfg._token}
@@ -671,10 +688,12 @@ class SASsessionHTTP():
          resp = req.read()
          conn.close()
 
-         js  = json.loads(resp.decode(self.sascfg.encoding))
-         lst = js.get('items')
-
-         lines = len(lst)
+         try:
+            js    = json.loads(resp.decode(self.sascfg.encoding))
+            lst   = js.get('items')
+            lines = len(lst)
+         except:
+            lines = None
 
          if not lines:
             break
@@ -807,41 +826,52 @@ class SASsessionHTTP():
             uri = ld.get('uri')
             break
 
-      conn    = self.sascfg.HTTPConn; conn.connect()
+      conn    = self.sascfg.HTTPConn;
       headers = {"Accept":"text/plain", "Authorization":"Bearer "+self.sascfg._token}
       done    = False
 
+      delay   = kwargs.get('GETstatusDelay'  , 0.5)
+      excpcnt = kwargs.get('GETstatusFailcnt',   5)
+
       while not done:
          try:
             while True:
                # GET Status for JOB
+               conn.connect()
                conn.request('GET', uri, headers=headers)
                req = conn.getresponse()
                resp = req.read()
+               conn.close()
                if resp not in [b'running', b'pending']:
                   done = True
                   break
-               sleep(.5)
+               sleep(delay)
+
          except (KeyboardInterrupt, SystemExit):
+            conn.close()
             print('Exception caught!')
             response = self.sascfg._prompt(
                       "SAS attention handling not yet supported over HTTP. Please enter (Q) to Quit waiting for results or (C) to continue waiting.")
             while True:
                if response is None or response.upper() == 'Q':
-                  conn.close()
                   return dict(LOG='', LST='', BC=True)
                if response.upper() == 'C':
                   break
                response = self.sascfg._prompt("Please enter (Q) to Quit waiting for results or (C) to continue waiting.")
 
-      conn.close()
+         except hc.RemoteDisconnected as Dis:
+            conn.close()
+            print('RemoteDisconnected Exception caught!\n'+str(Dis))
+            excpcnt -= 1
+            if excpcnt < 0:
+               raise
 
-      logd = self._getlog(jobid)
+      logd = self._getlog(jobid).replace(chr(12), chr(10))
 
       if ods:
-         lstd = self._getlst(jobid)
+         lstd = self._getlst(jobid).replace(chr(12), chr(10))
       else:
-         lstd = self._getlsttxt(jobid)
+         lstd = self._getlsttxt(jobid).replace(chr(12), chr(10))
 
       trip = lstd.rpartition("/*]]>*/")
       if len(trip[1]) > 0 and len(trip[2]) < 200:
@@ -1054,7 +1084,7 @@ class SASsessionHTTP():
 
          code = "filename _sp_updn;"
       else:
-         log1 = ''
+         logf = ''
          code = """
             filename _sp_updn '"""+remf+"""' recfm=F encoding=binary lrecl=1 permission='"""+permission+"""';
             data _null_;
@@ -1383,7 +1413,7 @@ class SASsessionHTTP():
       else:
          tabname = "'"+table.strip()+"'n "
 
-      code  = "data sasdata2dataframe / view=sasdata2dataframe; set "+tabname+self._sb._dsopts(dsopts)+";run;\n"
+      code  = "data work.sasdata2dataframe / view=work.sasdata2dataframe; set "+tabname+self._sb._dsopts(dsopts)+";run;\n"
 
       ll = self.submit(code, "text")
 
@@ -1420,12 +1450,13 @@ class SASsessionHTTP():
       topts.pop('firstobs', None)
       topts.pop('obs', None)
    
-      code  = "data work._n_u_l_l_;output;run;\n"
+      code  = "proc delete data=work.sasdata2dataframe(memtype=view);run;\n"
+      code += "data work._n_u_l_l_;output;run;\n"
       code += "data _null_; set work._n_u_l_l_ "+tabname+self._sb._dsopts(topts)+";put 'FMT_CATS=';\n"
    
       for i in range(nvars):
          code += "_tom = vformatn('"+varlist[i]+"'n);put _tom;\n"
-      code += "stop;\nrun;\n"
+      code += "stop;\nrun;\nproc delete data=work._n_u_l_l_;run;"
    
       ll = self.submit(code, "text")
 
@@ -1434,7 +1465,7 @@ class SASsessionHTTP():
       varcat = l2[2].split("\n", nvars)
       del varcat[nvars]
 
-      code  = "data work.saspy_ds2df / view=work.saspy_ds2df; set "+tabname+self._sb._dsopts(dsopts)+";\n"
+      code  = "data work.sasdata2dataframe / view=work.sasdata2dataframe; set "+tabname+self._sb._dsopts(dsopts)+";\n"
       for i in range(nvars):
          if vartype[i] == 'FLOAT':
             code += "format '"+varlist[i]+"'n "
@@ -1456,7 +1487,7 @@ class SASsessionHTTP():
 
       conn = self.sascfg.HTTPConn; conn.connect()
       headers={"Accept":"application/vnd.sas.collection+json", "Authorization":"Bearer "+self.sascfg._token}
-      uri = "/compute/sessions/"+self.pid+"/data/work/saspy_ds2df/rows"
+      uri = "/compute/sessions/"+self.pid+"/data/work/sasdata2dataframe/rows"
 
       r     = []
       df    = None
@@ -1530,6 +1561,8 @@ class SASsessionHTTP():
          else:
             df = tdf
 
+      ll = self.submit("proc delete data=work.sasdata2dataframe(memtype=view);run;", "text")
+
       return df
 
 
@@ -1566,7 +1599,7 @@ class SASsessionHTTP():
       else:
          tmpcsv  = tempfile
 
-      code  = "data sasdata2dataframe / view=sasdata2dataframe; set "+tabname+self._sb._dsopts(dsopts)+";run;\n"
+      code  = "data work.sasdata2dataframe / view=work.sasdata2dataframe; set "+tabname+self._sb._dsopts(dsopts)+";run;\n"
 
       ll = self.submit(code, "text")
 
@@ -1608,7 +1641,7 @@ class SASsessionHTTP():
    
       for i in range(nvars):
          code += "_tom = vformatn('"+varlist[i]+"'n);put _tom;\n"
-      code += "stop;\nrun;\n"
+      code += "stop;\nrun;\nproc delete data=work._n_u_l_l_;run;"
    
       ll = self.submit(code, "text")
 
@@ -1617,7 +1650,8 @@ class SASsessionHTTP():
       varcat = l2[2].split("\n", nvars)
       del varcat[nvars]
 
-      code  = "data work.sasdata2dataframe / view=work.sasdata2dataframe; set "+tabname+self._sb._dsopts(dsopts)+";\nformat "
+      code  = "proc delete data=work.sasdata2dataframe(memtype=view);run;\n"
+      code += "data work.sasdata2dataframe / view=work.sasdata2dataframe; set "+tabname+self._sb._dsopts(dsopts)+";\nformat "
 
       idx_col = kwargs.pop('index_col', False)
       eng     = kwargs.pop('engine',    'c')
@@ -1656,11 +1690,10 @@ class SASsessionHTTP():
       else:
          dts = k_dts
 
-      #code += "options nosource;\n"
       code  = "filename _tomodsx '"+self._sb.workpath+"_tomodsx' lrecl="+str(self.sascfg.lrecl)+" recfm=v  encoding='utf-8';\n"
       code += "proc export data=work.sasdata2dataframe outfile=_tomodsx dbms=csv replace;\n"
       code += self._sb._expopts(opts)+" run;\n"
-      #code += "options source;\n"
+      code += "proc delete data=work.sasdata2dataframe(memtype=view);run;\n"
 
       ll = self.submit(code, 'text')
 
@@ -1719,7 +1752,7 @@ class SASsessionHTTP():
       else:
          tmpcsv  = tempfile
 
-      code  = "data sasdata2dataframe / view=sasdata2dataframe; set "+tabname+self._sb._dsopts(dsopts)+";run;\n"
+      code  = "data work.sasdata2dataframe / view=work.sasdata2dataframe; set "+tabname+self._sb._dsopts(dsopts)+";run;\n"
 
       ll = self.submit(code, "text")
 
@@ -1756,12 +1789,13 @@ class SASsessionHTTP():
       topts.pop('firstobs', None)
       topts.pop('obs', None)
    
-      code  = "data work._n_u_l_l_;output;run;\n"
+      code  = "proc delete data=work.sasdata2dataframe(memtype=view);run;"
+      code += "data work._n_u_l_l_;output;run;\n"
       code += "data _null_; set work._n_u_l_l_ "+tabname+self._sb._dsopts(topts)+";put 'FMT_CATS=';\n"
    
       for i in range(nvars):
          code += "_tom = vformatn('"+varlist[i]+"'n);put _tom;\n"
-      code += "stop;\nrun;\n"
+      code += "stop;\nrun;\nproc delete data=work._n_u_l_l_;run;"
    
       ll = self.submit(code, "text")
 
